/* valaccodeassignmentmodule.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <gobject/valaccodeassignmentmodule.h>
#include <vala/valacodenode.h>
#include <vala/valacodevisitor.h>
#include <vala/valaexpression.h>
#include <vala/valaproperty.h>
#include <ccode/valaccodeexpression.h>
#include <vala/valamemberaccess.h>
#include <vala/valapropertyaccessor.h>
#include <vala/valaclass.h>
#include <vala/valatypesymbol.h>
#include <ccode/valaccodeconstant.h>
#include <vala/valadatatype.h>
#include <ccode/valaccodebinaryexpression.h>
#include <ccode/valaccodefunctioncall.h>
#include <vala/valaexpressionstatement.h>
#include <ccode/valaccodecommaexpression.h>
#include <vala/valaelementaccess.h>
#include <vala/valaarraytype.h>
#include <vala/valapointertype.h>
#include <gee/list.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vala/valasymbol.h>
#include <vala/valascope.h>
#include <stdlib.h>
#include <string.h>
#include <vala/valamethod.h>
#include <vala/valaformalparameter.h>
#include <vala/valagenerictype.h>
#include <vala/valasemanticanalyzer.h>
#include <ccode/valaccodeidentifier.h>
#include <ccode/valaccodecastexpression.h>
#include <vala/valareport.h>
#include <vala/valasourcereference.h>
#include <vala/valadelegatetype.h>
#include <vala/valadelegate.h>
#include <vala/valalocalvariable.h>
#include <ccode/valaccodeassignment.h>
#include <ccode/valaccodeelementaccess.h>
#include <gobject/valaccodebasemodule.h>




enum  {
	VALA_CCODE_ASSIGNMENT_MODULE_DUMMY_PROPERTY
};
static ValaCCodeExpression* vala_ccode_assignment_module_emit_property_assignment (ValaCCodeAssignmentModule* self, ValaAssignment* assignment);
static ValaCCodeExpression* vala_ccode_assignment_module_emit_non_array_element_access (ValaCCodeAssignmentModule* self, ValaAssignment* assignment);
static ValaCCodeExpression* vala_ccode_assignment_module_emit_simple_assignment (ValaCCodeAssignmentModule* self, ValaAssignment* assignment);
static void vala_ccode_assignment_module_real_visit_assignment (ValaCCodeModule* base, ValaAssignment* assignment);
static gpointer vala_ccode_assignment_module_parent_class = NULL;



ValaCCodeAssignmentModule* vala_ccode_assignment_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeAssignmentModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeAssignmentModule*) vala_ccode_member_access_module_construct (object_type, codegen, next);
	return self;
}


ValaCCodeAssignmentModule* vala_ccode_assignment_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_ccode_assignment_module_construct (VALA_TYPE_CCODE_ASSIGNMENT_MODULE, codegen, next);
}


static ValaCCodeExpression* vala_ccode_assignment_module_emit_property_assignment (ValaCCodeAssignmentModule* self, ValaAssignment* assignment) {
	ValaMemberAccess* _tmp1;
	ValaExpression* _tmp0;
	ValaMemberAccess* ma;
	ValaProperty* _tmp2;
	ValaProperty* prop;
	gboolean _tmp3;
	gboolean _tmp4;
	gboolean _tmp5;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (assignment != NULL, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	ma = (_tmp1 = (_tmp0 = vala_assignment_get_left (assignment), VALA_IS_MEMBER_ACCESS (_tmp0) ? ((ValaMemberAccess*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
	_tmp2 = NULL;
	prop = (_tmp2 = VALA_PROPERTY (vala_expression_get_symbol_reference (vala_assignment_get_left (assignment))), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
	_tmp3 = FALSE;
	_tmp4 = FALSE;
	_tmp5 = FALSE;
	if (vala_property_accessor_get_construction (vala_property_get_set_accessor (prop))) {
		_tmp5 = VALA_IS_CLASS (((ValaCCodeBaseModule*) self)->current_type_symbol);
	} else {
		_tmp5 = FALSE;
	}
	if (_tmp5) {
		_tmp4 = vala_typesymbol_is_subtype_of ((ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->current_class, ((ValaCCodeBaseModule*) self)->gobject_type);
	} else {
		_tmp4 = FALSE;
	}
	if (_tmp4) {
		_tmp3 = ((ValaCCodeBaseModule*) self)->in_creation_method;
	} else {
		_tmp3 = FALSE;
	}
	if (_tmp3) {
		ValaCCodeConstant* _tmp6;
		ValaCCodeExpression* _tmp7;
		ValaCCodeExpression* _tmp8;
		_tmp6 = NULL;
		_tmp7 = NULL;
		_tmp8 = NULL;
		return (_tmp8 = (_tmp7 = vala_ccode_module_get_construct_property_assignment (vala_ccode_module_get_head ((ValaCCodeModule*) self), _tmp6 = vala_property_get_canonical_cconstant (prop), vala_property_get_property_type (prop), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_assignment_get_right (assignment)))), (_tmp6 == NULL) ? NULL : (_tmp6 = (vala_ccode_node_unref (_tmp6), NULL)), _tmp7), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp8);
	} else {
		ValaCCodeExpression* _tmp9;
		ValaCCodeExpression* cexpr;
		ValaCCodeFunctionCall* ccall;
		_tmp9 = NULL;
		cexpr = (_tmp9 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_assignment_get_right (assignment))), (_tmp9 == NULL) ? NULL : vala_ccode_node_ref (_tmp9));
		if (!vala_property_get_no_accessor_method (prop)) {
			if (vala_data_type_is_real_struct_type (vala_property_get_property_type (prop))) {
				ValaCCodeExpression* _tmp10;
				_tmp10 = NULL;
				cexpr = (_tmp10 = vala_ccode_base_module_get_address_of_expression ((ValaCCodeBaseModule*) self, vala_assignment_get_right (assignment), cexpr), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp10);
			}
		}
		if (vala_assignment_get_operator (assignment) != VALA_ASSIGNMENT_OPERATOR_SIMPLE) {
			ValaCCodeBinaryOperator cop;
			ValaCCodeExpression* _tmp12;
			ValaCCodeExpression* _tmp11;
			cop = 0;
			if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
				cop = VALA_CCODE_BINARY_OPERATOR_BITWISE_OR;
			} else {
				if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
					cop = VALA_CCODE_BINARY_OPERATOR_BITWISE_AND;
				} else {
					if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
						cop = VALA_CCODE_BINARY_OPERATOR_BITWISE_XOR;
					} else {
						if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_ADD) {
							cop = VALA_CCODE_BINARY_OPERATOR_PLUS;
						} else {
							if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_SUB) {
								cop = VALA_CCODE_BINARY_OPERATOR_MINUS;
							} else {
								if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_MUL) {
									cop = VALA_CCODE_BINARY_OPERATOR_MUL;
								} else {
									if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_DIV) {
										cop = VALA_CCODE_BINARY_OPERATOR_DIV;
									} else {
										if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
											cop = VALA_CCODE_BINARY_OPERATOR_MOD;
										} else {
											if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
												cop = VALA_CCODE_BINARY_OPERATOR_SHIFT_LEFT;
											} else {
												if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
													cop = VALA_CCODE_BINARY_OPERATOR_SHIFT_RIGHT;
												} else {
													g_assert_not_reached ();
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			_tmp12 = NULL;
			_tmp11 = NULL;
			cexpr = (_tmp12 = (ValaCCodeExpression*) vala_ccode_binary_expression_new (cop, _tmp11 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_assignment_get_left (assignment))), cexpr), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp12);
			(_tmp11 == NULL) ? NULL : (_tmp11 = (vala_ccode_node_unref (_tmp11), NULL));
		}
		ccall = vala_ccode_base_module_get_property_set_call ((ValaCCodeBaseModule*) self, prop, ma, cexpr);
		/* assignments are expressions, so return the current property value, except if we're sure that it can't be used*/
		if (!(VALA_IS_EXPRESSION_STATEMENT (vala_code_node_get_parent_node ((ValaCodeNode*) assignment)))) {
			ValaCCodeCommaExpression* ccomma;
			ValaCCodeExpression* _tmp13;
			ValaCCodeExpression* _tmp14;
			ccomma = vala_ccode_comma_expression_new ();
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) ccall);
			/* update property*/
			_tmp13 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, _tmp13 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) ma)));
			(_tmp13 == NULL) ? NULL : (_tmp13 = (vala_ccode_node_unref (_tmp13), NULL));
			/* current property value*/
			_tmp14 = NULL;
			return (_tmp14 = (ValaCCodeExpression*) ccomma, (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), (ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp14);
		} else {
			ValaCCodeExpression* _tmp15;
			_tmp15 = NULL;
			return (_tmp15 = (ValaCCodeExpression*) ccall, (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), (ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL)), (prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL)), _tmp15);
		}
		(cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL));
		(ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL));
	}
	(ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL));
	(prop == NULL) ? NULL : (prop = (vala_code_node_unref (prop), NULL));
}


static ValaCCodeExpression* vala_ccode_assignment_module_emit_non_array_element_access (ValaCCodeAssignmentModule* self, ValaAssignment* assignment) {
	ValaCCodeExpression* _tmp0;
	ValaCCodeExpression* rhs;
	ValaElementAccess* _tmp1;
	ValaElementAccess* expr;
	ValaTypeSymbol* _tmp2;
	ValaTypeSymbol* container_type;
	GeeList* indices;
	GeeIterator* indices_it;
	ValaCCodeExpression* ccontainer;
	ValaExpression* _tmp3;
	ValaCCodeExpression* _tmp4;
	ValaCCodeExpression* cindex;
	gboolean _tmp5;
	gboolean _tmp6;
	gboolean _tmp7;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (assignment != NULL, NULL);
	/* custom element access*/
	_tmp0 = NULL;
	rhs = (_tmp0 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_assignment_get_right (assignment))), (_tmp0 == NULL) ? NULL : vala_ccode_node_ref (_tmp0));
	_tmp1 = NULL;
	expr = (_tmp1 = VALA_ELEMENT_ACCESS (vala_assignment_get_left (assignment)), (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
	_tmp2 = NULL;
	container_type = (_tmp2 = vala_data_type_get_data_type (vala_expression_get_value_type (vala_element_access_get_container (expr))), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
	indices = vala_element_access_get_indices (expr);
	indices_it = gee_iterable_iterator ((GeeIterable*) indices);
	gee_iterator_next (indices_it);
	ccontainer = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_element_access_get_container (expr)));
	_tmp3 = NULL;
	_tmp4 = NULL;
	cindex = (_tmp4 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) (_tmp3 = (ValaExpression*) gee_iterator_get (indices_it)))), (_tmp3 == NULL) ? NULL : (_tmp3 = (vala_code_node_unref (_tmp3), NULL)), _tmp4);
	_tmp5 = FALSE;
	_tmp6 = FALSE;
	_tmp7 = FALSE;
	if (container_type != NULL) {
		_tmp7 = ((ValaCCodeBaseModule*) self)->list_type != NULL;
	} else {
		_tmp7 = FALSE;
	}
	if (_tmp7) {
		_tmp6 = ((ValaCCodeBaseModule*) self)->map_type != NULL;
	} else {
		_tmp6 = FALSE;
	}
	if (_tmp6) {
		gboolean _tmp8;
		_tmp8 = FALSE;
		if (vala_typesymbol_is_subtype_of (container_type, (ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->list_type)) {
			_tmp8 = TRUE;
		} else {
			_tmp8 = vala_typesymbol_is_subtype_of (container_type, (ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->map_type);
		}
		_tmp5 = (_tmp8);
	} else {
		_tmp5 = FALSE;
	}
	if (_tmp5) {
		ValaTypeSymbol* collection_iface;
		ValaMethod* set_method;
		GeeList* set_params;
		GeeIterator* set_params_it;
		ValaFormalParameter* set_param;
		ValaCCodeIdentifier* _tmp15;
		char* _tmp14;
		ValaCCodeFunctionCall* _tmp16;
		ValaCCodeFunctionCall* set_ccall;
		ValaCCodeCastExpression* _tmp19;
		char* _tmp18;
		char* _tmp17;
		ValaCCodeExpression* _tmp20;
		ValaCCodeExpression* _tmp21;
		/* lookup symbol in interface instead of class as implemented interface methods are not in VAPI files*/
		collection_iface = NULL;
		if (vala_typesymbol_is_subtype_of (container_type, (ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->list_type)) {
			ValaTypeSymbol* _tmp10;
			ValaTypeSymbol* _tmp9;
			_tmp10 = NULL;
			_tmp9 = NULL;
			collection_iface = (_tmp10 = (_tmp9 = (ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->list_type, (_tmp9 == NULL) ? NULL : vala_code_node_ref (_tmp9)), (collection_iface == NULL) ? NULL : (collection_iface = (vala_code_node_unref (collection_iface), NULL)), _tmp10);
		} else {
			if (vala_typesymbol_is_subtype_of (container_type, (ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->map_type)) {
				ValaTypeSymbol* _tmp12;
				ValaTypeSymbol* _tmp11;
				_tmp12 = NULL;
				_tmp11 = NULL;
				collection_iface = (_tmp12 = (_tmp11 = (ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->map_type, (_tmp11 == NULL) ? NULL : vala_code_node_ref (_tmp11)), (collection_iface == NULL) ? NULL : (collection_iface = (vala_code_node_unref (collection_iface), NULL)), _tmp12);
			}
		}
		set_method = VALA_METHOD (vala_scope_lookup (vala_symbol_get_scope ((ValaSymbol*) collection_iface), "set"));
		set_params = vala_method_get_parameters (set_method);
		set_params_it = gee_iterable_iterator ((GeeIterable*) set_params);
		gee_iterator_next (set_params_it);
		set_param = (ValaFormalParameter*) gee_iterator_get (set_params_it);
		if (VALA_IS_GENERIC_TYPE (vala_formal_parameter_get_parameter_type (set_param))) {
			ValaDataType* index_type;
			ValaCCodeExpression* _tmp13;
			index_type = vala_semantic_analyzer_get_actual_type (vala_expression_get_value_type (vala_element_access_get_container (expr)), VALA_GENERIC_TYPE (vala_formal_parameter_get_parameter_type (set_param)), (ValaCodeNode*) assignment);
			_tmp13 = NULL;
			cindex = (_tmp13 = vala_ccode_base_module_convert_to_generic_pointer ((ValaCCodeBaseModule*) self, cindex, index_type), (cindex == NULL) ? NULL : (cindex = (vala_ccode_node_unref (cindex), NULL)), _tmp13);
			(index_type == NULL) ? NULL : (index_type = (vala_code_node_unref (index_type), NULL));
		}
		_tmp15 = NULL;
		_tmp14 = NULL;
		_tmp16 = NULL;
		set_ccall = (_tmp16 = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp15 = vala_ccode_identifier_new (_tmp14 = vala_method_get_cname (set_method)))), (_tmp15 == NULL) ? NULL : (_tmp15 = (vala_ccode_node_unref (_tmp15), NULL)), _tmp14 = (g_free (_tmp14), NULL), _tmp16);
		_tmp19 = NULL;
		_tmp18 = NULL;
		_tmp17 = NULL;
		vala_ccode_function_call_add_argument (set_ccall, (ValaCCodeExpression*) (_tmp19 = vala_ccode_cast_expression_new (ccontainer, _tmp18 = g_strconcat (_tmp17 = vala_typesymbol_get_cname (collection_iface, FALSE), "*", NULL))));
		(_tmp19 == NULL) ? NULL : (_tmp19 = (vala_ccode_node_unref (_tmp19), NULL));
		_tmp18 = (g_free (_tmp18), NULL);
		_tmp17 = (g_free (_tmp17), NULL);
		vala_ccode_function_call_add_argument (set_ccall, cindex);
		_tmp20 = NULL;
		vala_ccode_function_call_add_argument (set_ccall, _tmp20 = vala_ccode_base_module_convert_to_generic_pointer ((ValaCCodeBaseModule*) self, rhs, vala_expression_get_value_type ((ValaExpression*) expr)));
		(_tmp20 == NULL) ? NULL : (_tmp20 = (vala_ccode_node_unref (_tmp20), NULL));
		_tmp21 = NULL;
		return (_tmp21 = (ValaCCodeExpression*) set_ccall, (collection_iface == NULL) ? NULL : (collection_iface = (vala_code_node_unref (collection_iface), NULL)), (set_method == NULL) ? NULL : (set_method = (vala_code_node_unref (set_method), NULL)), (set_params == NULL) ? NULL : (set_params = (gee_collection_object_unref (set_params), NULL)), (set_params_it == NULL) ? NULL : (set_params_it = (gee_collection_object_unref (set_params_it), NULL)), (set_param == NULL) ? NULL : (set_param = (vala_code_node_unref (set_param), NULL)), (rhs == NULL) ? NULL : (rhs = (vala_ccode_node_unref (rhs), NULL)), (expr == NULL) ? NULL : (expr = (vala_code_node_unref (expr), NULL)), (container_type == NULL) ? NULL : (container_type = (vala_code_node_unref (container_type), NULL)), (indices == NULL) ? NULL : (indices = (gee_collection_object_unref (indices), NULL)), (indices_it == NULL) ? NULL : (indices_it = (gee_collection_object_unref (indices_it), NULL)), (ccontainer == NULL) ? NULL : (ccontainer = (vala_ccode_node_unref (ccontainer), NULL)), (cindex == NULL) ? NULL : (cindex = (vala_ccode_node_unref (cindex), NULL)), _tmp21);
	} else {
		ValaCCodeExpression* _tmp22;
		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) assignment), "internal error: unsupported element access");
		vala_code_node_set_error ((ValaCodeNode*) assignment, TRUE);
		_tmp22 = NULL;
		return (_tmp22 = NULL, (rhs == NULL) ? NULL : (rhs = (vala_ccode_node_unref (rhs), NULL)), (expr == NULL) ? NULL : (expr = (vala_code_node_unref (expr), NULL)), (container_type == NULL) ? NULL : (container_type = (vala_code_node_unref (container_type), NULL)), (indices == NULL) ? NULL : (indices = (gee_collection_object_unref (indices), NULL)), (indices_it == NULL) ? NULL : (indices_it = (gee_collection_object_unref (indices_it), NULL)), (ccontainer == NULL) ? NULL : (ccontainer = (vala_ccode_node_unref (ccontainer), NULL)), (cindex == NULL) ? NULL : (cindex = (vala_ccode_node_unref (cindex), NULL)), _tmp22);
	}
	(rhs == NULL) ? NULL : (rhs = (vala_ccode_node_unref (rhs), NULL));
	(expr == NULL) ? NULL : (expr = (vala_code_node_unref (expr), NULL));
	(container_type == NULL) ? NULL : (container_type = (vala_code_node_unref (container_type), NULL));
	(indices == NULL) ? NULL : (indices = (gee_collection_object_unref (indices), NULL));
	(indices_it == NULL) ? NULL : (indices_it = (gee_collection_object_unref (indices_it), NULL));
	(ccontainer == NULL) ? NULL : (ccontainer = (vala_ccode_node_unref (ccontainer), NULL));
	(cindex == NULL) ? NULL : (cindex = (vala_ccode_node_unref (cindex), NULL));
}


static ValaCCodeExpression* vala_ccode_assignment_module_emit_simple_assignment (ValaCCodeAssignmentModule* self, ValaAssignment* assignment) {
	ValaCCodeExpression* _tmp0;
	ValaCCodeExpression* rhs;
	gboolean unref_old;
	gboolean array;
	gboolean instance_delegate;
	gboolean _tmp3;
	gboolean _tmp4;
	ValaCCodeAssignmentOperator cop;
	ValaCCodeExpression* _tmp15;
	ValaCCodeExpression* _tmp16;
	ValaCCodeExpression* codenode;
	gboolean _tmp17;
	ValaCCodeExpression* _tmp25;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (assignment != NULL, NULL);
	_tmp0 = NULL;
	rhs = (_tmp0 = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_assignment_get_right (assignment))), (_tmp0 == NULL) ? NULL : vala_ccode_node_ref (_tmp0));
	unref_old = vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_assignment_get_left (assignment)));
	array = FALSE;
	instance_delegate = FALSE;
	if (VALA_IS_ARRAY_TYPE (vala_expression_get_value_type (vala_assignment_get_left (assignment)))) {
		ValaCCodeExpression* _tmp1;
		_tmp1 = NULL;
		array = !(VALA_IS_CCODE_CONSTANT (_tmp1 = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_assignment_get_left (assignment), 1)));
		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_ccode_node_unref (_tmp1), NULL));
	} else {
		if (VALA_IS_DELEGATE_TYPE (vala_expression_get_value_type (vala_assignment_get_left (assignment)))) {
			ValaDelegateType* _tmp2;
			ValaDelegateType* delegate_type;
			_tmp2 = NULL;
			delegate_type = (_tmp2 = VALA_DELEGATE_TYPE (vala_expression_get_value_type (vala_assignment_get_left (assignment))), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
			instance_delegate = vala_delegate_get_has_target (vala_delegate_type_get_delegate_symbol (delegate_type));
			(delegate_type == NULL) ? NULL : (delegate_type = (vala_code_node_unref (delegate_type), NULL));
		}
	}
	_tmp3 = FALSE;
	_tmp4 = FALSE;
	if (unref_old) {
		_tmp4 = TRUE;
	} else {
		_tmp4 = array;
	}
	if (_tmp4) {
		_tmp3 = TRUE;
	} else {
		_tmp3 = instance_delegate;
	}
	if (_tmp3) {
		ValaCCodeCommaExpression* ccomma;
		ValaLocalVariable* temp_decl;
		ValaCCodeAssignment* _tmp6;
		ValaCCodeIdentifier* _tmp5;
		ValaCCodeIdentifier* _tmp12;
		ValaCCodeExpression* _tmp14;
		ValaCCodeExpression* _tmp13;
		ccomma = vala_ccode_comma_expression_new ();
		temp_decl = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_assignment_get_left (assignment)), TRUE, NULL);
		gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_decl);
		_tmp6 = NULL;
		_tmp5 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp6 = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp5 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) temp_decl))), rhs, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		(_tmp6 == NULL) ? NULL : (_tmp6 = (vala_ccode_node_unref (_tmp6), NULL));
		(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_ccode_node_unref (_tmp5), NULL));
		if (unref_old) {
			ValaCCodeExpression* _tmp8;
			ValaCCodeExpression* _tmp7;
			/* unref old value */
			_tmp8 = NULL;
			_tmp7 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, _tmp8 = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, _tmp7 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_assignment_get_left (assignment))), vala_expression_get_value_type (vala_assignment_get_left (assignment)), vala_assignment_get_left (assignment)));
			(_tmp8 == NULL) ? NULL : (_tmp8 = (vala_ccode_node_unref (_tmp8), NULL));
			(_tmp7 == NULL) ? NULL : (_tmp7 = (vala_ccode_node_unref (_tmp7), NULL));
		}
		if (array) {
			ValaArrayType* _tmp9;
			ValaArrayType* array_type;
			_tmp9 = NULL;
			array_type = (_tmp9 = VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_assignment_get_left (assignment))), (_tmp9 == NULL) ? NULL : vala_code_node_ref (_tmp9));
			{
				gint dim;
				dim = 1;
				for (; dim <= vala_array_type_get_rank (array_type); dim++) {
					ValaCCodeExpression* lhs_array_len;
					ValaCCodeExpression* rhs_array_len;
					ValaCCodeAssignment* _tmp10;
					lhs_array_len = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_assignment_get_left (assignment), dim);
					rhs_array_len = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_assignment_get_right (assignment), dim);
					_tmp10 = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp10 = vala_ccode_assignment_new (lhs_array_len, rhs_array_len, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					(_tmp10 == NULL) ? NULL : (_tmp10 = (vala_ccode_node_unref (_tmp10), NULL));
					(lhs_array_len == NULL) ? NULL : (lhs_array_len = (vala_ccode_node_unref (lhs_array_len), NULL));
					(rhs_array_len == NULL) ? NULL : (rhs_array_len = (vala_ccode_node_unref (rhs_array_len), NULL));
				}
			}
			(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
		} else {
			if (instance_delegate) {
				ValaCCodeExpression* lhs_delegate_target;
				ValaCCodeExpression* rhs_delegate_target;
				ValaCCodeAssignment* _tmp11;
				lhs_delegate_target = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, vala_assignment_get_left (assignment));
				rhs_delegate_target = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, vala_assignment_get_right (assignment));
				_tmp11 = NULL;
				vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp11 = vala_ccode_assignment_new (lhs_delegate_target, rhs_delegate_target, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
				(_tmp11 == NULL) ? NULL : (_tmp11 = (vala_ccode_node_unref (_tmp11), NULL));
				(lhs_delegate_target == NULL) ? NULL : (lhs_delegate_target = (vala_ccode_node_unref (lhs_delegate_target), NULL));
				(rhs_delegate_target == NULL) ? NULL : (rhs_delegate_target = (vala_ccode_node_unref (rhs_delegate_target), NULL));
			}
		}
		_tmp12 = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp12 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) temp_decl))));
		(_tmp12 == NULL) ? NULL : (_tmp12 = (vala_ccode_node_unref (_tmp12), NULL));
		_tmp14 = NULL;
		_tmp13 = NULL;
		rhs = (_tmp14 = (_tmp13 = (ValaCCodeExpression*) ccomma, (_tmp13 == NULL) ? NULL : vala_ccode_node_ref (_tmp13)), (rhs == NULL) ? NULL : (rhs = (vala_ccode_node_unref (rhs), NULL)), _tmp14);
		(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
		(temp_decl == NULL) ? NULL : (temp_decl = (vala_code_node_unref (temp_decl), NULL));
	}
	cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE;
	if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_BITWISE_OR) {
		cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_OR;
	} else {
		if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_BITWISE_AND) {
			cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_AND;
		} else {
			if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_BITWISE_XOR) {
				cop = VALA_CCODE_ASSIGNMENT_OPERATOR_BITWISE_XOR;
			} else {
				if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_ADD) {
					cop = VALA_CCODE_ASSIGNMENT_OPERATOR_ADD;
				} else {
					if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_SUB) {
						cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SUB;
					} else {
						if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_MUL) {
							cop = VALA_CCODE_ASSIGNMENT_OPERATOR_MUL;
						} else {
							if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_DIV) {
								cop = VALA_CCODE_ASSIGNMENT_OPERATOR_DIV;
							} else {
								if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_PERCENT) {
									cop = VALA_CCODE_ASSIGNMENT_OPERATOR_PERCENT;
								} else {
									if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_SHIFT_LEFT) {
										cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_LEFT;
									} else {
										if (vala_assignment_get_operator (assignment) == VALA_ASSIGNMENT_OPERATOR_SHIFT_RIGHT) {
											cop = VALA_CCODE_ASSIGNMENT_OPERATOR_SHIFT_RIGHT;
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	_tmp15 = NULL;
	_tmp16 = NULL;
	codenode = (_tmp16 = (ValaCCodeExpression*) vala_ccode_assignment_new (_tmp15 = VALA_CCODE_EXPRESSION (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_assignment_get_left (assignment))), rhs, cop), (_tmp15 == NULL) ? NULL : (_tmp15 = (vala_ccode_node_unref (_tmp15), NULL)), _tmp16);
	_tmp17 = FALSE;
	if (unref_old) {
		ValaCCodeNode* _tmp18;
		_tmp18 = NULL;
		_tmp17 = VALA_IS_CCODE_ELEMENT_ACCESS (_tmp18 = vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_assignment_get_left (assignment)));
		(_tmp18 == NULL) ? NULL : (_tmp18 = (vala_ccode_node_unref (_tmp18), NULL));
	} else {
		_tmp17 = FALSE;
	}
	if (_tmp17) {
		ValaCCodeElementAccess* cea;
		gboolean _tmp19;
		/* ensure that index expression in element access doesn't get evaluated more than once
		 except when it's a simple expression*/
		cea = VALA_CCODE_ELEMENT_ACCESS (vala_ccode_base_module_get_ccodenode ((ValaCCodeBaseModule*) self, (ValaCodeNode*) vala_assignment_get_left (assignment)));
		_tmp19 = FALSE;
		if (VALA_IS_CCODE_CONSTANT (vala_ccode_element_access_get_index (cea))) {
			_tmp19 = TRUE;
		} else {
			_tmp19 = VALA_IS_CCODE_IDENTIFIER (vala_ccode_element_access_get_index (cea));
		}
		if (!(_tmp19)) {
			ValaLocalVariable* index_temp_decl;
			ValaCCodeCommaExpression* ccomma;
			ValaCCodeAssignment* _tmp21;
			ValaCCodeIdentifier* _tmp20;
			ValaCCodeIdentifier* _tmp22;
			ValaCCodeExpression* _tmp24;
			ValaCCodeExpression* _tmp23;
			index_temp_decl = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL);
			gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, index_temp_decl);
			ccomma = vala_ccode_comma_expression_new ();
			_tmp21 = NULL;
			_tmp20 = NULL;
			vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp21 = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp20 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) index_temp_decl))), vala_ccode_element_access_get_index (cea), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
			(_tmp21 == NULL) ? NULL : (_tmp21 = (vala_ccode_node_unref (_tmp21), NULL));
			(_tmp20 == NULL) ? NULL : (_tmp20 = (vala_ccode_node_unref (_tmp20), NULL));
			vala_ccode_comma_expression_append_expression (ccomma, codenode);
			_tmp22 = NULL;
			vala_ccode_element_access_set_index (cea, (ValaCCodeExpression*) (_tmp22 = vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) index_temp_decl))));
			(_tmp22 == NULL) ? NULL : (_tmp22 = (vala_ccode_node_unref (_tmp22), NULL));
			_tmp24 = NULL;
			_tmp23 = NULL;
			codenode = (_tmp24 = (_tmp23 = (ValaCCodeExpression*) ccomma, (_tmp23 == NULL) ? NULL : vala_ccode_node_ref (_tmp23)), (codenode == NULL) ? NULL : (codenode = (vala_ccode_node_unref (codenode), NULL)), _tmp24);
			(index_temp_decl == NULL) ? NULL : (index_temp_decl = (vala_code_node_unref (index_temp_decl), NULL));
			(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
		}
		(cea == NULL) ? NULL : (cea = (vala_ccode_node_unref (cea), NULL));
	}
	_tmp25 = NULL;
	return (_tmp25 = codenode, (rhs == NULL) ? NULL : (rhs = (vala_ccode_node_unref (rhs), NULL)), _tmp25);
}


static void vala_ccode_assignment_module_real_visit_assignment (ValaCCodeModule* base, ValaAssignment* assignment) {
	ValaCCodeAssignmentModule * self;
	gboolean _tmp0;
	self = (ValaCCodeAssignmentModule*) base;
	g_return_if_fail (assignment != NULL);
	vala_code_node_accept ((ValaCodeNode*) vala_assignment_get_right (assignment), (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	_tmp0 = FALSE;
	if (vala_code_node_get_error ((ValaCodeNode*) vala_assignment_get_left (assignment))) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = vala_code_node_get_error ((ValaCodeNode*) vala_assignment_get_right (assignment));
	}
	if (_tmp0) {
		vala_code_node_set_error ((ValaCodeNode*) assignment, TRUE);
		return;
	}
	if (VALA_IS_PROPERTY (vala_expression_get_symbol_reference (vala_assignment_get_left (assignment)))) {
		ValaCCodeExpression* _tmp1;
		_tmp1 = NULL;
		vala_code_node_set_ccodenode ((ValaCodeNode*) assignment, (ValaCCodeNode*) (_tmp1 = vala_ccode_assignment_module_emit_property_assignment (self, assignment)));
		(_tmp1 == NULL) ? NULL : (_tmp1 = (vala_ccode_node_unref (_tmp1), NULL));
	} else {
		gboolean _tmp2;
		gboolean _tmp3;
		_tmp2 = FALSE;
		_tmp3 = FALSE;
		if (VALA_IS_ELEMENT_ACCESS (vala_assignment_get_left (assignment))) {
			_tmp3 = !(VALA_IS_ARRAY_TYPE (vala_expression_get_value_type (vala_element_access_get_container ((VALA_ELEMENT_ACCESS (vala_assignment_get_left (assignment)))))));
		} else {
			_tmp3 = FALSE;
		}
		if (_tmp3) {
			_tmp2 = !(VALA_IS_POINTER_TYPE (vala_expression_get_value_type (vala_element_access_get_container ((VALA_ELEMENT_ACCESS (vala_assignment_get_left (assignment)))))));
		} else {
			_tmp2 = FALSE;
		}
		if (_tmp2) {
			ValaCCodeExpression* _tmp4;
			_tmp4 = NULL;
			vala_code_node_set_ccodenode ((ValaCodeNode*) assignment, (ValaCCodeNode*) (_tmp4 = vala_ccode_assignment_module_emit_non_array_element_access (self, assignment)));
			(_tmp4 == NULL) ? NULL : (_tmp4 = (vala_ccode_node_unref (_tmp4), NULL));
		} else {
			ValaCCodeExpression* _tmp5;
			_tmp5 = NULL;
			vala_code_node_set_ccodenode ((ValaCodeNode*) assignment, (ValaCCodeNode*) (_tmp5 = vala_ccode_assignment_module_emit_simple_assignment (self, assignment)));
			(_tmp5 == NULL) ? NULL : (_tmp5 = (vala_ccode_node_unref (_tmp5), NULL));
		}
	}
}


static void vala_ccode_assignment_module_class_init (ValaCCodeAssignmentModuleClass * klass) {
	vala_ccode_assignment_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->visit_assignment = vala_ccode_assignment_module_real_visit_assignment;
}


static void vala_ccode_assignment_module_instance_init (ValaCCodeAssignmentModule * self) {
}


GType vala_ccode_assignment_module_get_type (void) {
	static GType vala_ccode_assignment_module_type_id = 0;
	if (vala_ccode_assignment_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeAssignmentModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_assignment_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeAssignmentModule), 0, (GInstanceInitFunc) vala_ccode_assignment_module_instance_init, NULL };
		vala_ccode_assignment_module_type_id = g_type_register_static (VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, "ValaCCodeAssignmentModule", &g_define_type_info, 0);
	}
	return vala_ccode_assignment_module_type_id;
}




